% *
% * RemoveSegmentesThromboBorders.m
% *
% *      Author: neeb
% 
% Copyright 2013 Heiko Neeb
% 
% This file is part of the Grnwald Remagen OpenSource Unified 
% Platelet-Identification Tracker (GROUP-IT) toolkit.
% 
% GROUP-IT is free software: you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation, either version 3 of the License, or
% (at your option) any later version.
% 
% GROUP-IT is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
% 
% You should have received a copy of the GNU General Public License
% along with GROUP-IT.  If not, see <http://www.gnu.org/licenses/>.
% 
% GROUP-IT is designed to be used for scientific purposes only. Do NOT
% base any diagnosis on the results obtained with this program


function [ ThromboMask ] = RemoveSegmentedThromboBorders( ThromboMask, borderCutoff, scale, minArea, maxArea )

    % square scale as we are interested in area
    scale=scale^2;

    % "maximum" grey value in perliminarily segmented image
    maxGW=quantile(ThromboMask(ThromboMask>0),0.95);
    
    % remove voxels with gw comparable to the thrombo edge and small
    % objects
    [ThroMaskLabel,nThromb]=bwlabel(ThromboMask>0);
    for(TCnt=1:nThromb)
        curLabelImg=ThromboMask;
        curLabelImg((ThroMaskLabel~=TCnt))=0;
        
%        curLabelSize=length(curLabelImg(curLabelImg>0))*scale;
%        if(curLabelSize<minArea | curLabelSize>maxArea ) 
%            ThromboMask(curLabelImg>0)=0; 
%            continue;
%        end
        
        ed=edge(double(curLabelImg>0));
        tt=curLabelImg(ed>0);
        ThromboMask(curLabelImg>0)=0;
        curLabelImg(curLabelImg<borderCutoff*mean(tt(tt>0)) & curLabelImg<0.9*max(curLabelImg(:)))=0;
        ThromboMask=ThromboMask+curLabelImg;
    end
    
    % Remove the remaining small or large objects
    % Furthermore, remove object with small and rather homogeneous grey value
    % distribution
    SE=strel('disk',1);
    ThromboMask=imerode(imfill(ThromboMask),SE);
    [ThroMaskLabel,nThromb]=bwlabel(ThromboMask>0);
    for(TCnt=1:nThromb)
        curLabelImg=ThromboMask;
        curLabelImg((ThroMaskLabel~=TCnt))=0;
        curLabelSize=length(curLabelImg(curLabelImg>0))*scale;
        
        curLabelMaxGW=quantile(curLabelImg(curLabelImg>0),0.95);
        if(curLabelSize<minArea | curLabelSize>maxArea | curLabelMaxGW<0.6*maxGW) 
            ThromboMask(curLabelImg>0)=0;
            continue;
        end
    end

end

