% *
% * SimpleThromboTimeseriesTracker.m
% *
% *      Author: neeb
% 
% Copyright 2013 Heiko Neeb
% 
% This file is part of the Grnwald Remagen OpenSource Unified 
% Platelet-Identification Tracker (GROUP-IT) toolkit.
% 
% GROUP-IT is free software: you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation, either version 3 of the License, or
% (at your option) any later version.
% 
% GROUP-IT is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
% 
% You should have received a copy of the GNU General Public License
% along with GROUP-IT.  If not, see <http://www.gnu.org/licenses/>.
% 
% GROUP-IT is designed to be used for scientific purposes only. Do NOT
% base any diagnosis on the results obtained with this program


DataSeriesName='Timelapse_klein';
DataSetID='26743012';

Folder=sprintf('/Users/neeb/data/Thrombocytes/SystematicMeasurements/%s/%s_ResultSegments/',DataSeriesName,DataSetID);

loaded=0;
plateletsTracked=0;
parametersDetermined=0;
resultsStored=0;

% recon parameters
minOverlapp=0.3;
exclusionWidth=5;
trackingSearchRadius=20; %Pixel
LeukocyteMinGrayVal=70;
filenameExtension='.tiff';


% slope and offset parameters for grouping points based on scatter between
% FD and perimeter    
s0=0.0643; o0=0.7642; s1=0.0225; o1=0.825; s2=0.0078; o2=0.844;


%% load thrombo images from current folder
if( loaded == 0 )
    [originalSegments, fileNames] = loadThromboFolder(Folder, -1 );
end    
    
nTP=size(originalSegments,3);


%% Track platelets in timeseries and remove all fast moving objects
if( plateletsTracked == 0 )
    
    segments=originalSegments;

    % loop over all images in stack
    for(i=exclusionWidth+1:nTP-exclusionWidth-1)
    %for(i=201:201)

        i
        % label current segment                    
        curSegment=segments(:,:,i);
        [curImageLabeled, nThrombos]=bwlabel(curSegment);


        for(thro=1:nThrombos)

            curLabMask=curImageLabeled==thro;       
            if(length(curLabMask(curLabMask>0)) == 0 )
                continue;
            end

            originalLabelSize=length(curLabMask(curLabMask>0));

            % multiply with the previous and following segments
            for(excl=-exclusionWidth:exclusionWidth)
                curLabMaskRed=curLabMask.*originalSegments(:,:,i+excl);
            end
            
            newLabelSize=length(curLabMaskRed(curLabMaskRed>0));

            % check if segment has to be removed from image
            if( newLabelSize<minOverlapp*originalLabelSize)
                segments(:,:,i)=segments(:,:,i)-curLabMask;
            end
        end            
    end
end


%% Determine Thrombo Parameters
if( parametersDetermined == 0 ) 

    %LabNr BildNr YY MM DD HH MM SS x y	  Area 	Perimeter   FD  CirC  AvGW
    %  1     2    3  4  5  6  7  8  9 10   11       12      13   14    15
    
    % StdGW Eccent EquivDia Extent MajorAxisL MinorAxisL Solidity
    %  16     17     18      19       20          21        22

    % nPseudoPods DiameterBody  AreaPseudoPods DiameterPseudo DistPseudo
    %     23              24        25              26            27

    % scatterGroupID  K:n(F) K:y_quer  K:sum_y_i  K:a_quer  K:sum_y_i	
    %     28           29     30          31         32         33
        
    % K:e_quer  K:sum_e_i  K:d
    %     34       35      36  


   
    % Those are the parameters in the manual analysis table
    % ID     Nr
    %  1     2

    %LabNr BildNr YY  MM DD  HH MM  SS   x   y	Area Peri   FD  CirC AvGW 
    %  3     4     5   6  7  8   9  10  11  12   13   14    15   16   17

    % n(F)	y_quer	sum_y_i	a_quer	sum_y_i	e_quer	sum_e_i	  d
    %  18     19       20    21       22      23       24     25

    % determine border lines between scatter groups
    scatterGroupSlopeOffset=zeros(3,2);
    scatterGroupSlopeOffset(1,1)=s0;
    scatterGroupSlopeOffset(1,2)=o0;
    scatterGroupSlopeOffset(2,1)=s1;
    scatterGroupSlopeOffset(2,2)=o1;
    scatterGroupSlopeOffset(3,1)=s2;
    scatterGroupSlopeOffset(3,2)=o2;

    % load scale info
    ScaleFile=sprintf('/Users/neeb/data/Thrombocytes/SystematicMeasurements/%s/%s_Results/ScaleInfo.txt',DataSeriesName,DataSetID);
    scale = dlmread(ScaleFile);

    % determine parameters for segments which were remaining after tracking
    for(i=1:nTP)
%    for(i=40:40 ) 

        i 
        
        % get parameters
        SeriesID=str2num(DataSetID(end-4:end-1));
        [curthromboResults,cnt, segments(:,:,i) ]=determineThromboParameters( segments(:,:,i), scale, 1, LeukocyteMinGrayVal,...
            scatterGroupSlopeOffset, 0, fileNames(i).name, filenameExtension, SeriesID);
        
        % combine matrices
        if( i == 1 )
            thromboResults=curthromboResults;
        else
            thromboResults=vertcat(thromboResults,curthromboResults);
        end
    end
    
    
    % Now sort results according to each individual platelet. The first
    % colum of the corresponding result matrix now contains the running ID
    % of the platelet instead of the LabNr.

    % take an image id as reference and loop over all segments in this image
    refID=round(min(thromboResults(:,2))+0.75*(max(thromboResults(:,2))-min(thromboResults(:,2))));

    refResults=thromboResults(thromboResults(:,2)==refID,:);
    nThrombInRef=size(refResults(:,1));
    for(i=1:nThrombInRef)

        % search the corresponding segments in the other images
        curX=refResults(i,9);
        curY=refResults(i,10);
        distance=sqrt((thromboResults(:,9)-curX).^2+(thromboResults(:,10)-curY).^2);

        sameSegmentAtDifferentTP=thromboResults(distance<=trackingSearchRadius,:);
        sameSegmentAtDifferentTP(:,1)=i;

        % combine matrices
        if( i == 1 )
            thromboResultsSorted=sameSegmentAtDifferentTP;
        else
            thromboResultsSorted=vertcat(thromboResultsSorted,sameSegmentAtDifferentTP);
        end
    end

    
    % clean up images and remove all segments which are not contained in the
    % thromboResultsSorted list (from the segment images)
    firstImgID=str2num(fileNames(1).name(1:4)); % ID of the first image in the current series
    
    for(i=1:nTP)

        refImage=segments(:,:,refID-firstImgID+1);


        % label corresponding image
        [curImageLabeled, nThrombos]=bwlabel(segments(:,:,i));

        effectiveSegments=curImageLabeled.*refImage;
        for(thro=1:nThrombos)
            if(length(effectiveSegments(effectiveSegments==thro)) == 0 )
                segments(:,:,i)=segments(:,:,i)-(curImageLabeled==thro);
                continue;
            end
        end
    end
    
end


%% Store results
if( resultsStored == 0 )
    ResultFolder=sprintf('/Users/neeb/data/Thrombocytes/SystematicMeasurements/%s/%s_ResultSegments_Tracked/',DataSeriesName,DataSetID);
    for(i=1:nTP)
                   
        filename=fileNames(i).name;
        filename=strrep(filename,filenameExtension,'');

        %imwrite(uint8(binarySegments(:,:,i)),sprintf('/Users/neeb/data/Thrombocytes/Results/new/%s/%s.tiff',...
        %    DataSetID,filename),'tif');
        
        imwrite(uint8(segments(:,:,i)),sprintf('%s/%s.tiff',...
            ResultFolder,filename),'tif');
    end
    
    % Store sorted result matrix
    dlmwrite(sprintf('%s/ResultMatrix_Tracked.txt', ResultFolder), ...
        thromboResultsSorted,'delimiter', '\t', 'precision', '%4.3f');
    
    %store timeevolution of FD-Perim Scatter plots
    minImageID=min(thromboResultsSorted(thromboResultsSorted(:,2)>0,2))
    maxImageID=max(thromboResultsSorted(:,2))

    cnt=0;
    for(id=minImageID:maxImageID)

        ScatterFolder=sprintf('/Users/neeb/data/Thrombocytes/SystematicMeasurements/%s/%s_ResultSegments_Perim_VS_FD/',DataSeriesName,DataSetID);
        curParamMat=thromboResultsSorted(thromboResultsSorted(:,2)==id,:);
        cnt=cnt+1;

        % some scatter plots
        close all
        figure, scatter(curParamMat(:,12),curParamMat(:,13),190,curParamMat(:,11),'fill')   
        xlabel('Perimeter [\mum]', 'FontSize',20)
        ylabel('Fractal Dimension', 'FontSize',20)
        set(gca,'fontsize',20);  

        % add grouping info
        x=0:0.1:50; x1=0:0.1:90; x0=0:0.1:20; hold on;
        plot(x1,s2*x1+o2,'g');
        plot(x,s1*x+o1,'r');
        plot(x0,s0*x0+o0,'k');
        %saveas(gcf,sprintf('/Users/neeb/data/Thrombocytes/Results/new/%s/Perimeter_VS_FD.pdf',...
        %    DataSetID),'pdf');
        saveas(gcf,sprintf('%s/Perimeter_VS_FD_%04i.jpg', ScatterFolder,id),'jpg');

    end
    
end

