% *
% * ThromboAnalysiBatch.m
% *
% *      Author: neeb
% 
% Copyright 2013 Heiko Neeb
% 
% This file is part of the Grnwald Remagen OpenSource Unified 
% Platelet-Identification Tracker (GROUP-IT) toolkit.
% 
% GROUP-IT is free software: you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation, either version 3 of the License, or
% (at your option) any later version.
% 
% GROUP-IT is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
% 
% You should have received a copy of the GNU General Public License
% along with GROUP-IT.  If not, see <http://www.gnu.org/licenses/>.
% 
% GROUP-IT is designed to be used for scientific purposes only. Do NOT
% base any diagnosis on the results obtained with this program


DataSeriesName='';
DataSetID='26743102'; %3003
RanomiseDataID=0;

Folder=sprintf('/Users/neeb/data/Thrombocytes/SystematicMeasurements/%s/%s/',DataSeriesName,DataSetID);
%Folder=sprintf('/Users/neeb/data/Thrombocytes/new/26743620/');

loaded=1;
thrombosSegmented=0;
parametersDetermined=0;
resultsStored=1;
checkGrouping=0;
storeIndivudualThromboImages=0;
determinePseudoPods=0;
PSFdetermined=1;
identifyAgglomerates=1;

maxNumOfImagesToProcess=50;

% recon parameters
filenameExtension='.JPG';
borderCutoff=1.2;  %1.2 % for timeseries: 1.6

if( thrombosSegmented==0 )
    %scale=1/27.3; %[um/pixel]   
%    scale=1/7.8;  %[um/pixel]
    scale=-1;
end
scaleBarDist=10; %um
minThromArea=5; % 10 um^2
maxThromArea=450;  % 490 um^2
individThromboStoreProbability=0.15;
LeukocyteMinGrayVal=70;
TargetImageSizeX=720; %Pixel

ShrinkAxis=1;

% slope and offset parameters for grouping points based on scatter between
% FD and perimeter    
s0=0.0643; o0=0.7642; s1=0.0225; o1=0.825; s2=0.0078; o2=0.844;



%% load thrombo images from current folder
if( loaded == 0 )
    [ThromStack, fileNames] = loadThromboFolder(Folder, TargetImageSizeX );
    
    
    % check if external scale info is given
    if( scale <= 0)
        nImages=size(ThromStack,3)-1;
    
        % get Ruler
        Ruler=ThromStack(:,:,nImages+1);
        ThromStack=ThromStack(:,:,1:nImages);
    else
        nImages=size(ThromStack,3);
    end
    
    
    % check if image has to be shrunken
    if(ShrinkAxis>1)
        % calulate new image dimensions
        nRowNew=floor(size(ThromStack,1)/ShrinkAxis);
        nColNew=floor(size(ThromStack,2)/ShrinkAxis);
        
        ThromStackNew=zeros(nRowNew,nColNew,nImages);
        
        % loop over all images
        for(z=1:nImages)
            z
            for(y=1:nRowNew)
                for(x=1:nColNew)
                    tmp=ThromStack((y-1)*ShrinkAxis+1:y*ShrinkAxis-1,(x-1)*ShrinkAxis+1:x*ShrinkAxis,z);
                    ThromStackNew(y,x,z)=mean(tmp(:));
                end
            end
        end
        
        ThromStack=ThromStackNew;
        
    end
    
    % Load Result Matrix from manual evaluation
    load('/Users/neeb/data/Thrombocytes/Results/new/SelectedIndividualSegmentInfo/TotalResults.mat');
end


%% ESTIMATE THE POINT-SPREAD-FUNCTION
if( PSFdetermined == 0 )

    % Estimate average PSF
    AvgPSF=zeros(150,1);
    IndividPSF=zeros(150,nImages);
    INITPSF = ones(150,1);
    xx=1:150;
    for(i=1:nImages)        
        i
        curImage=ThromStack(:,:,i);
        [curImage, PSF]=deconvblind(curImage,INITPSF,10);
        IndividPSF(:,i)=PSF;
        AvgPSF=AvgPSF+PSF;
    end        
    AvgPSF=AvgPSF/nImages;
    PSFWidth=IndividPSF'*((xx-75).^2)';
    PSFsforAveraging=IndividPSF(:,abs(PSFWidth-median(PSFWidth))<0.5*median(PSFWidth));
    AvgSelectedPSF=mean(PSFsforAveraging');
end


%% SEGMENT
if( thrombosSegmented == 0 ) 
    
    % determine proper scale
    if( scale <= 0 )
        rulerProj=sum(Ruler(:,floor(size(Ruler,2)/2)-5:floor(size(Ruler,2)/2)+5)');
        rulerProj(rulerProj<0.5*max(rulerProj))=0;
        rulerProj(rulerProj>0)=1;
        scaleBars=find(diff(rulerProj)>0);
        scale=ShrinkAxis*scaleBarDist/(scaleBars(3)-scaleBars(1));
    end
    
    % loop over all images and segment
    segments=zeros(size(ThromStack));
    nRow=size(ThromStack,1);
    nCol=size(ThromStack,2);
    borderImage=zeros(nRow,nCol);
    borderImage(1,1:nCol)=1; borderImage(nRow,1:nCol)=1;
    borderImage(1:nRow,1)=1; borderImage(1:nRow,nCol)=1;    
        
    for(i=1:nImages)
        %if(i~=472)
        %    continue;
        %end

        % check if there is a maximum number of images to be processed
        % specified by the user
        if(maxNumOfImagesToProcess > 0 && i>maxNumOfImagesToProcess )
            continue;
        end
        
        i
        curImage=ThromStack(:,:,i);
        
        % NEW FOR TESTING!!!!
        [curImage]=deconvlucy(curImage, AvgSelectedPSF);
        
        % segment
        [curImageSeg]=segmentThrombos(curImage,4,0);
        
        % remove borders and objects which are too small or too large
        [curImageSegCleaned]=RemoveSegmentedThromboBorders( curImageSeg, ...
            borderCutoff, scale, minThromArea, maxThromArea );
                
        % remove thrombos which are located at the image border
        [curLabelImg,nLabel]=bwlabel(curImageSegCleaned);
        tmp=curLabelImg.*borderImage;
        labelHist=histc(tmp(tmp>0),1:nLabel);
        borderLabels=find(labelHist>0);
        for(label=1:length(borderLabels))
            curImageSegCleaned(curLabelImg==borderLabels(label))=0;
        end
        segments(:,:,i)=curImageSegCleaned;
        

%        figure, imagesc(curImageSegCleaned)
%        close all

    end
end


%% Determine Thrombo Parameters
if( parametersDetermined == 0 ) 

    %LabNr BildNr YY MM DD HH MM SS x y	  Area 	Perimeter   FD  CirC  AvGW
    %  1     2    3  4  5  6  7  8  9 10   11       12      13   14    15
    
    % StdGW Eccent EquivDia Extent MajorAxisL MinorAxisL Solidity
    %  16     17     18      19       20          21        22

    % nPseudoPods DiameterBody  AreaPseudoPods DiameterPseudo DistPseudo
    %     23              24        25              26            27

    % scatterGroupID  K:n(F) K:y_quer  K:sum_y_i  K:a_quer  K:sum_y_i	
    %     28           29     30          31         32         33
        
    % K:e_quer  K:sum_e_i  K:d
    %     34       35      36  


   
    % Those are the parameters in the manual analysis table
    % ID     Nr
    %  1     2

    %LabNr BildNr YY  MM DD  HH MM  SS   x   y	Area Peri   FD  CirC AvGW 
    %  3     4     5   6  7  8   9  10  11  12   13   14    15   16   17

    % n(F)	y_quer	sum_y_i	a_quer	sum_a_i	e_quer	sum_e_i	  d
    %  18     19       20    21       22      23       24     25

    
    if( checkGrouping > 0 )
        Group1=zeros(size(segments));
        Group2=zeros(size(segments));
        Group3=zeros(size(segments));
    end
    
    if( determinePseudoPods > 0 )
        ImagesWithPseudopods=zeros(size(segments));
    end
    
    % determine border lines between scatter groups
    scatterGroupSlopeOffset=zeros(3,2);
    scatterGroupSlopeOffset(1,1)=s0;
    scatterGroupSlopeOffset(1,2)=o0;
    scatterGroupSlopeOffset(2,1)=s1;
    scatterGroupSlopeOffset(2,2)=o1;
    scatterGroupSlopeOffset(3,1)=s2;
    scatterGroupSlopeOffset(3,2)=o2;

    for(i=1:nImages)
%    for(i=40:40 ) 

        % check if there is a maximum number of images to be processed
        % specified by the user
        if(maxNumOfImagesToProcess > 0 && i>maxNumOfImagesToProcess )
            continue;
        end

        i 
        
        % get parameters
        SeriesID=str2num(Folder(end-4:end-1));
        [curthromboResults,cnt, segments(:,:,i) ]=determineThromboParameters( segments(:,:,i), scale, 1, LeukocyteMinGrayVal,...
            scatterGroupSlopeOffset, 0, fileNames(i).name, filenameExtension, SeriesID);
        
        for(k=1:cnt)
            % Attach results from manual characterisation if available
            TotalResultForCurSeg=TotalResult(TotalResult(:,3)==curthromboResults(k,1),:);
            TotalResultForCurSeg=TotalResultForCurSeg(TotalResultForCurSeg(:,4)==curthromboResults(k,2),:);
            TotalResultForCurSeg=TotalResultForCurSeg(...
                abs(TotalResultForCurSeg(:,11)-curthromboResults(k,9))<2,:);
            TotalResultForCurSeg=TotalResultForCurSeg(...
                abs(TotalResultForCurSeg(:,12)-curthromboResults(k,10))<2,:);
            
            if( size(TotalResultForCurSeg,1) > 0 )
                curthromboResults(k,29:36)=TotalResultForCurSeg(1,18:25);
            end
            
%             % combine matrices
%             if( i == 1 )
%                 thromboResults=curthromboResults;
%             else
%                 thromboResults=vertcat(thromboResults,curthromboResults);
%             end
            
            % check grouping of segments in three distinct groups
            if( checkGrouping > 0 )
                
                % check the group where current thrombo belongs to
                curLabMask=bwlabel(segments(:,:,i));

                for(k=1:cnt);
                    scatterGroupID=curthromboResults(k,28);

                    x=curthromboResults(k,9);
                    y=curthromboResults(k,10);
                    tmpSegment=segments(:,:,i);
                    tmpSegment(curLabMask ~= curLabMask(y,x))=0;
                
                    if( scatterGroupID==1 )
                        Group1(:,:,i)=Group1(:,:,i)+tmpSegment;
                    end
                    if( scatterGroupID==2 )
                        Group2(:,:,i)=Group2(:,:,i)+tmpSegment;
                    end
                    if( scatterGroupID==3 )
                        Group3(:,:,i)=Group3(:,:,i)+tmpSegment;
                    end
                end
            end
        end
        
        % combine matrices
        if( i == 1 )
            thromboResults=curthromboResults;
        else
            thromboResults=vertcat(thromboResults,curthromboResults);
        end
    end
end




%% Store results
if(resultsStored == 0 )


    binarySegments=segments;
    binarySegments(binarySegments>0)=1;


    ResultFolder=sprintf('/Users/neeb/data/Thrombocytes/SystematicMeasurements/%s/%s_Results/',DataSeriesName,DataSetID);
    for(i=1:nImages)
                   
        filename=fileNames(i).name;
        filename=strrep(filename,filenameExtension,'');

        %imwrite(uint8(binarySegments(:,:,i)),sprintf('/Users/neeb/data/Thrombocytes/Results/new/%s/%s.tiff',...
        %    DataSetID,filename),'tif');
        
        imwrite(uint8(binarySegments(:,:,i)),sprintf('%s/%s.tiff',...
            ResultFolder,filename),'tif');
    end
    clear binarySegments;
    
    if( checkGrouping > 0 )
         figure;
         for(i=1:nImages)
             filename=fileNames(i).name;
             filename=strrep(filename,filenameExtension,'');
             
             imagesc(Group1(:,:,i),[0 110])
             saveas(gcf,sprintf('/Users/neeb/data/Thrombocytes/Results/new/%s_Subgroups/Group1/%s.jpg',...
                 DataSetID,filename),'jpg');
                          
             imagesc(Group2(:,:,i),[0 110])
             saveas(gcf,sprintf('/Users/neeb/data/Thrombocytes/Results/new/%s_Subgroups/Group2/%s.jpg',...
                 DataSetID,filename),'jpg');

             imagesc(Group3(:,:,i),[0 110])
             saveas(gcf,sprintf('/Users/neeb/data/Thrombocytes/Results/new/%s_Subgroups/Group3/%s.jpg',...
                 DataSetID,filename),'jpg');
         end
    end
    % some scatter plots
    close all
    figure, scatter(thromboResults(:,12),thromboResults(:,13)); 
    xlabel('Perimeter [\mum]', 'FontSize',20)
    ylabel('Fractal Dimension', 'FontSize',20)
    set(gca,'fontsize',20);  
    
    % add grouping info
    x=0:0.1:50; x1=0:0.1:90; x0=0:0.1:20; hold on;
    plot(x1,s2*x1+o2,'g');
    plot(x,s1*x+o1,'r');
    plot(x0,s0*x0+o0,'k');
    %saveas(gcf,sprintf('/Users/neeb/data/Thrombocytes/Results/new/%s/Perimeter_VS_FD.pdf',...
    %    DataSetID),'pdf');
    saveas(gcf,sprintf('%s/Perimeter_VS_FD.pdf', ResultFolder),'pdf');

    close all
    figure;
    
    scatter(thromboResults(:,12),thromboResults(:,11)); 
    xlabel('Perimeter [\mum]', 'FontSize',20)
    ylabel('Area [\mum^2]', 'FontSize',20)
    set(gca,'fontsize',20);  
    %saveas(gcf,sprintf('/Users/neeb/data/Thrombocytes/Results/new/%s/Perimeter_VS_Area.pdf',...
    %    DataSetID),'pdf');
    saveas(gcf,sprintf('%s/Perimeter_VS_Area.pdf', ResultFolder),'pdf');

    scatter(thromboResults(:,13),thromboResults(:,11)); 
    xlabel('Fractal Dimension', 'FontSize',20)
    ylabel('Area [\mum^2]', 'FontSize',20)
    set(gca,'fontsize',20);  
    %saveas(gcf,sprintf('/Users/neeb/data/Thrombocytes/Results/new/%s/FD_VS_Area.pdf',...
    %    DataSetID),'pdf');
    saveas(gcf,sprintf('%s/FD_VS_Area.pdf', ResultFolder),'pdf');

    % store matrices
    %dlmwrite(sprintf('/Users/neeb/data/Thrombocytes/Results/new/%s/ResultMatrix.txt', DataSetID), ...
    %    thromboResults,'delimiter', '\t', 'precision', '%4.3f');
    dlmwrite(sprintf('%s/ResultMatrix.txt', ResultFolder), ...
        thromboResults,'delimiter', '\t', 'precision', '%4.3f');

    % store scale information (might be used later)
    dlmwrite(sprintf('%s/ScaleInfo.txt', ResultFolder), ...
        scale,'delimiter', '\t', 'precision', '%10.6f');

end



%% Store indivudual thrombo outlines
if( storeIndivudualThromboImages > 0 )
    
    figure;
   cnt=0;
   realCnt=0;
   selectedthromboResultsFull=zeros(1,17);
   selectedthromboResults=zeros(1,2);
   for(i=1:nImages)
        
       % label current segment                
        curSegment=segments(:,:,i);
        [curImageLabeled, nThrombos]=bwlabel(curSegment);
        
        for(thro=1:nThrombos)

            
            curLabMask=curImageLabeled==thro;       
            if(length(curLabMask(curLabMask>0)) == 0 )
                continue;
            end
            
            % check if current thrombo contains multiple segments. If so,
            % continue
            tmpLabel=bwlabel(curLabMask);
            if(max(tmpLabel(:))>1 )
                continue;
            end

            cnt=cnt+1;
            
            % check if thrombo should be stored
            if( rand > individThromboStoreProbability )
                continue;
            end
            
            % store results
            realCnt=realCnt+1;
            selectedthromboResultsFull(realCnt,3:17)=thromboResults(cnt,:);
            selectedthromboResultsFull(realCnt,1)=RanomiseDataID;
            selectedthromboResultsFull(realCnt,2)=realCnt;

            selectedthromboResults(realCnt,1)=RanomiseDataID;
            selectedthromboResults(realCnt,2)=realCnt;

            
            STATS = regionprops(curLabMask, 'BoundingBox');
            ULCorner=max([1 1], floor(STATS.BoundingBox(1:2)-[2 2]));
            RegExtend=ceil(STATS.BoundingBox(3:4)+[4 4]);
            LRCorner=min( [ULCorner(1)+RegExtend(1) ULCorner(2)+RegExtend(2)], [size(curLabMask,2) size(curLabMask,1)]);
            
            curThrombo=curLabMask(ULCorner(2):LRCorner(2),ULCorner(1):LRCorner(1));
            curThrombo=edge(imfill(curThrombo,'holes'),'log');
            curSegmentFull=ThromStack(:,:,i);
            curThromboImage=curSegmentFull(ULCorner(2):LRCorner(2),ULCorner(1):LRCorner(1));
            curThromboImage(curThrombo>0)=80;
            
            % embed in standard thrombo frame
            standardThromboTemplate=zeros(400,400);
            standardThromboImage=zeros(400,400);
            curThromboRows=size(curThrombo,1);
            curThromboCols=size(curThrombo,2);
            if( mod(curThromboRows,2) == 0 )
                % even
                lowerBoundRow=200-curThromboRows/2;
                upperBoundRow=200+curThromboRows/2-1;
            else
                % odd
                lowerBoundRow=200-(curThromboRows-1)/2;
                upperBoundRow=200+(curThromboRows-1)/2;
            end
            if( mod(curThromboCols,2) == 0 )
                % even
                lowerBoundCol=200-curThromboCols/2;
                upperBoundCol=200+curThromboCols/2-1;
            else
                % odd
                lowerBoundCol=200-(curThromboCols-1)/2;
                upperBoundCol=200+(curThromboCols-1)/2;
            end
            standardThromboTemplate(lowerBoundRow:upperBoundRow,...
                lowerBoundCol:upperBoundCol)=curThrombo;
            standardThromboImage(lowerBoundRow:upperBoundRow,...
                lowerBoundCol:upperBoundCol)=curThromboImage;
            
            %            imagesc( standardThromboTemplate ); colormap('gray');
            filename=fileNames(i).name;
            filename=strrep(filename,filenameExtension,'');

            % position of current thrombocyte
            xpos=ceil(1*(ULCorner(1)+RegExtend(1)/2));
            ypos=ceil(1*(ULCorner(2)+RegExtend(2)/2));

%            dlmwrite(sprintf('/Users/neeb/data/Thrombocytes/Results/new/IndividualSegments/ID%i_Nr%i.txt',RanomiseDataID,realCnt),...
%                standardThromboTemplate,'delimiter', ' ', 'precision', '%i');
%            imagesc( standardThromboImage ); %colormap('gray')
%            saveas(gcf,sprintf('/Users/neeb/data/Thrombocytes/Results/new/IndividualSegmentsJPG/ID%i_Nr%i.jpg',...
%                    RanomiseDataID,realCnt),'jpg');
            standardThromboImage=round(standardThromboImage);
            standardThromboImage=uint8(standardThromboImage);
            imwrite(standardThromboImage,jet,sprintf('/Users/neeb/data/Thrombocytes/Results/new/IndividualSegmentsJPG/ID%i_Nr%i.png',...
                    RanomiseDataID,realCnt),'png');
        end
   end
   
   % store result matrices
   dlmwrite(sprintf('/Users/neeb/data/Thrombocytes/Results/new/SelectedIndividualSegmentInfo/FullResults_%s_DataID%i.txt', DataSetID,RanomiseDataID), ...
        selectedthromboResultsFull,'delimiter', '\t', 'precision', '%4.3f');

   dlmwrite(sprintf('/Users/neeb/data/Thrombocytes/Results/new/SelectedIndividualSegmentInfo/ShortResults_DataID%i.txt', RanomiseDataID), ...
        selectedthromboResults,'delimiter', '\t', 'precision', '%4.3f');
end


% Check if Agglomerates should be identified
if( identifyAgglomerates > 0 )
    
    % edge detection filter mask
    sobelMask = fspecial('sobel');
    SE=strel('disk',1);

    % Loop over all images in stack
    for(i=1:size(ThromStack,3))
%    for(i=1:1)
        
        % Determine edges for all identified segments
        %origninalEdges = segments(:,:,i).*ThromStack(:,:,i)/max(segments(:));
        curSegmentMask = segments(:,:,i)>0;
        outerEdges=edge(curSegmentMask);
        outerEdges=imdilate(outerEdges,SE);
        origninalEdges = curSegmentMask.*ThromStack(:,:,i);
        origninalEdges = sqrt(imfilter(origninalEdges,sobelMask).^2+imfilter(origninalEdges,sobelMask').^2);
        
        % remove the outer edges from image
        origninalEdges(outerEdges>0)=0;        
        
        %figure, imagesc(origninalEdges,[0 100]); colormap('gray')
        
        % set 'non-edges' to zero
        origninalEdges(origninalEdges<0.6*quantile(origninalEdges(origninalEdges>0),0.9))=0;
        origninalEdges=imfill(origninalEdges,'holes');
        
        % label and remove remaining small objects
        tmpLabel=bwlabel(origninalEdges>0);
        numLabels=max(tmpLabel(:));

        figure, imagesc(origninalEdges,[0 100]); colormap('gray')
        for(j=1:numLabels)
            curRegion=tmpLabel==j;
            curLabelSize=length(curRegion(curRegion>0))*scale^2;
            if(curLabelSize<minThromArea ) 
                curLabelImg=tmpLabel==j;
                origninalEdges(curLabelImg==1)=0;
            end
        end

        %figure, imagesc(origninalEdges,[0 100]); colormap('gray')
    end
    
end


