%% Function for segmentation of Thrombocytes
function [ThromboMask]=segmentThrombos(reducedRGB, biasFieldCorStrength, verbose)

dim=size(reducedRGB);
reducedRGBOrig=reducedRGB;
reducedRGB=anisodiff2D(reducedRGB, 10, 1/7, 20, 1);
ThromboMask=zeros(dim);

%% calculate low frequency bias field and substract form original image
% First, filter original image to suppress noise
if( biasFieldCorStrength > 0 )
    
    % Calculate low freq bias field (variable "a")
    at=reducedRGB;    
    for(i=1:biasFieldCorStrength)
        at(at>(1.5-(i-1)/10)*quantile(at(:),0.3))=0;
        Background=estimateBackground(at, 10, 10);
        at(at==0)=Background(at==0);
        if( verbose > 2 )
           figure, imagesc(at)
        end
    end

    % Filter Bias Field
    h = fspecial('gaussian', 30, 30);
    a=imfilter(at,h,'replicate');
    
    % subtact bias 
    aa=reducedRGB-a;
    aa(aa>0.5*max(aa(:)))=0; % get rid of writing in image
    aa=imfill(aa,'holes');
    aa(aa<0)=0;
    if( verbose > 2 )
        figure, imagesc(a)
        figure, imagesc(aa)
    end
else
    aa=reducedRGB;
end


%% Now we start the basic segmentation
tmp1=aa>0.03*max(aa(:));

if( verbose > 2 )
    figure, imagesc(tmp1)
end
tmpLabel=bwlabel(tmp1);
numLabels=max(tmpLabel(:));

% check if there are too many labels. If so, perform opening first
if(numLabels>600)
    se=strel('disk',2);
    tmp1=imopen(tmp1,se);
    tmpLabel=bwlabel(tmp1);
    numLabels=max(tmpLabel(:));
end

for(i=1:numLabels)
    curRegion=tmpLabel==i;
    curLabelSize=length(curRegion(curRegion>0));
    curCentreSize=aa(curRegion>0);
    curCentreSize=length(curCentreSize(curCentreSize>0.1*max(aa(:))));
    if(curLabelSize<800 && curCentreSize<50 ) 
        curLabelImg=tmpLabel==i;
        tmp1(curLabelImg==1)=0;
    end
end
tmp=reducedRGBOrig;
tmp(tmp1==0)=0;
%figure, imagesc(tmp,[0 350]); colormap('gray')

ThromboMask=tmp;

end



%% Estimate background field
function [Background]=estimateBackground(original, numX, numY)

dim=size(original);
Background=zeros(dim);
sizeX=floor(dim(2)/numX);
sizeY=floor(dim(1)/numY);

for(x=1:numX)
    for(y=1:numY)
        curMat=original((y-1)*sizeY+1:y*sizeY,(x-1)*sizeX+1:x*sizeX);
        if(sum(curMat(:)) == 0)
            Background((y-1)*sizeY+1:y*sizeY,(x-1)*sizeX+1:x*sizeX)=0; 
        else
            Background((y-1)*sizeY+1:y*sizeY,(x-1)*sizeX+1:x*sizeX)=median(curMat(curMat~=0)); 
        end
    end
end

Background(Background==0)=median(Background(:));
h = fspecial('gaussian', floor(sizeX/2), floor(sizeX/2));
Background=imfilter(Background,h,'replicate');
Background(Background==0)=median(Background(Background>0));

end
